package com.raycos.raycosdatacenterapi.authorize;

import com.alibaba.fastjson2.JSONObject;
import com.raycos.raycoscommon.Internal.Result;
import com.raycos.raycoscommon.Internal.datacenterapi.TbAmzToken;
import com.raycos.raycoscommon.Internal.datacenterapi.vo.GetAmzTokenByLwaCodeVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 亚马逊令牌 服务接口
 */
@RestController
@Slf4j
@RequestMapping("/erp/amztoken")
public class AmzTokenController {

    @Autowired
    private AmzTokenService amzTokenService;

    @GetMapping
    public Result<List<TbAmzToken>> selToken() {
        Result<List<TbAmzToken>> tr = new Result<>();
        try {
            List<TbAmzToken> list = amzTokenService.selAmzToken(null);
            tr.success(list);
        } catch (Exception e) {
            tr.fail();
        }
        return tr;
    }

    /**
     * 新增 amz令牌
     *
     * @param tbAmzToken
     * @return
     */
    @PostMapping
    public Result<String> addToken(@RequestBody TbAmzToken tbAmzToken) {
        Result<String> rt = new Result<>();
        try {
            log.info("  新增 token   来了老弟  " + JSONObject.toJSONString(tbAmzToken));
            amzTokenService.addAmzToken(tbAmzToken);
            rt.success();
        } catch (Exception e) {
            log.error(" 新增token 异常 ", e);
            rt.fail();
        }
        return rt;

    }

    /**
     * LWA code 获取亚马逊令牌
     *
     * @return
     */
    @PostMapping("/getamztokenbylwacode")
    public Result<String> getAmzTokenByLwaCode(@RequestBody GetAmzTokenByLwaCodeVo params) {
        Result<String> rt = new Result<>();
        try {
            Assert.hasLength(params.getLwaCode() , " getLwaCode is Empty 请检查提交参数 ！");
            Assert.notNull(params.getStoreId() , " getStoreId is Empty 请检查提交参数 ！");
            Assert.notNull(params.getCompanyId() , " getCompanyId is Empty 请检查提交参数 ！");
            Assert.hasLength(params.getSellingPartnerId() , " getSellingPartnerId is Empty 请检查提交参数 ！");

            rt = amzTokenService.getAmzTokenByLwaCode( params);

        } catch (Exception e) {
            rt.fail();
            rt.setMsg( e.getMessage());
            log.error(" /getamztokenbylwacode 异常 ", e);
        }
        return rt;
    }

    /**
     * 刷新 LWA  令牌
     *
     * @return
     */
    @GetMapping("/lrt")
    public Result<String> lwaRefreshToken(@RequestBody TbAmzToken tbAmzToken) {
        Result<String> rt = new Result<>();
        try {
            amzTokenService.lwaRefreshToken(tbAmzToken);
            rt.success();
        } catch (Exception e) {
            rt.fail();
            log.error(" 新增token 异常 ", e);
        }
        return rt;
    }


}
