package com.raycos.raycosdatacenterapi.scheduledtasks;

import com.raycos.raycoscommon.Internal.datacenterapi.TbAmzToken;
import com.raycos.raycosdatacenterapi.authorize.AmzTokenService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @Author: xiaojiabin
 * @Date: 2024/9/2 16:12
 */

@Component
@Slf4j
public class RefreshAmazonToken {

    /**
     * {
     * "access_token": "Atza|IwEBIBfvDi8BGuMZaWHUiClkd6mr9gucUJlIyf4yFjxpao9cfC4jDZXoud2G3ktVcMz6bYw2aweBLUu1E3otBYSaFbY7mJjqKGanMIW0EQX2a-RHPjpx_zkdkv719DmvZquQDgRCXywCogNG81usXd-ZwFYBQ96XjcYPaTC8w4WNNdJ4JHAo02BHRzLCV7I5E-S2u6PPoeM8lv-k7QXDvQhw7S1X2ldI2CVN0QYzMTDUqQ8M7Sn-UcaUJX_CD1AgN1HbsnJZlYLcC1FVmN5rj0tNzqA0s7Usc3be7RPVtFzBa58h75H7QnETHuhn_VordCETb7g",
     * "refresh_token": "Atzr|IwEBIITKfZoBv6fJBrBBG1tJHhgCxqePUah2uQpaUaCRgMpzYowNrpO1fQZqJuH-YxtXue5E6-0fWlQWKr9BfxuyfnCejJtaX3jxKo0TdN6l6woAv8awQrhLJuwO-KN3kDrdiB3VKyEDy-R6V0pKz7b9w0gzTN-PEL-e7uA-5CaRLOwV_299HF4ZTsz5nrfZIQMZ1S0Rgbs3TNHhw48OAlLOIXlwt95mPL7gMLvy6CJf-u5_vwh38sCQLu472ViwGGr936lGN38QSBaXaUf6mKt9LT_12C5x68nBGNsi1ZhzdMGDLJZcVHiq5CSXzL-HSa1G2VDv-0ej1YupcjBefCFBGnIS",
     * "token_type": "bearer",
     * "expires_in": 3600
     * }
     */
    private static final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("HH:mm:ss");

    @Autowired
    private AmzTokenService amzTokenService;
    @Scheduled(fixedRate = 5000)
    public void reportCurrentTime() {

        log.info("现在时间是：" + dateTimeFormatter.format(LocalDateTime.now()));

        try{
            // 查询 令牌
            List<TbAmzToken> list = amzTokenService.selAmzToken(null);
            if(CollectionUtils.isEmpty(list)){
                log.info(" 没有查询到 需要 刷新的 amz token 稍后再试！     ");
                return;
            }
            // 获取当前时间戳
            long second = Instant.now().getEpochSecond();

            // 检查是否临近过期时间
            for (TbAmzToken tbAmzToken : list) {

                Long expiresIn = tbAmzToken.getExpiresIn();
                if(  second >=  expiresIn ){
                    RefreshTask.queue.put( tbAmzToken );
                }

            }
        }catch (Exception e){
            log.error( e.getMessage() , e );
        }


    }




}
