package com.raycos.raycosdatacenterapi.scheduledtasks;

import com.alibaba.fastjson2.JSONObject;
import com.raycos.raycoscommon.Internal.datacenterapi.TbAmzToken;
import com.raycos.raycosdatacenterapi.authorize.AmzTokenService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedTransferQueue;
import java.util.concurrent.locks.ReentrantLock;

@Component
@Slf4j
public class RefreshTask {

    public static LinkedTransferQueue<TbAmzToken> queue = new LinkedTransferQueue<>();
    private final ExecutorService singleThreadPool = Executors.newSingleThreadExecutor();

    @Autowired
    private AmzTokenService amzTokenService;



    public static final ReentrantLock lock = new ReentrantLock();


    @Scheduled(fixedRate = 5000)
    public void runTask() {
        try {

            log.info(" 调用用亚马逊 刷新令牌接口    ");
            if (queue.isEmpty()) {
                log.info(" 暂时没有需要刷新的LWA 令牌    ");
                return;
            }
            if (lock.isLocked()) {
                log.info(" 执行中...    ");
                return;
            }
            lock.lock();
            Runnable runnable;
            while (!queue.isEmpty()) {

                // 实现线程
                runnable = new Runnable() {
                    @Override
                    public void run() {
                        TbAmzToken poll = queue.poll();
                        try {
                            amzTokenService.lwaRefreshToken(  poll );
                        } catch (Exception e) {
                            log.error(" 亚马逊刷新令牌程序出错   异常参数=>" + JSONObject.toJSONString(poll) , e);
                        }

                    }
                };
                singleThreadPool.execute(runnable);
                // 每个发出一个刷新请求 睡眠300毫秒 避免发送太快 超出亚马逊 每秒最大5个请求的限制
                Thread.sleep(300);

            }




        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try{
                lock.unlock();
            }catch (Exception ee){

            }
        }

    }






    /*
    {
    "grant_type": "refresh_token",
    "refresh_token": "Atzr|IwEBIITKfZoBv6fJBrBBG1tJHhgCxqePUah2uQpaUaCRgMpzYowNrpO1fQZqJuH-YxtXue5E6-0fWlQWKr9BfxuyfnCejJtaX3jxKo0TdN6l6woAv8awQrhLJuwO-KN3kDrdiB3VKyEDy-R6V0pKz7b9w0gzTN-PEL-e7uA-5CaRLOwV_299HF4ZTsz5nrfZIQMZ1S0Rgbs3TNHhw48OAlLOIXlwt95mPL7gMLvy6CJf-u5_vwh38sCQLu472ViwGGr936lGN38QSBaXaUf6mKt9LT_12C5x68nBGNsi1ZhzdMGDLJZcVHiq5CSXzL-HSa1G2VDv-0ej1YupcjBefCFBGnIS",
    "client_id": "amzn1.application-oa2-client.2d001c4aeaaf438f98ee0b07070c1cf8",
    "client_secret": "amzn1.oa2-cs.v1.52b13f4ce04e83046b3426b620b6bfb94fd3ce61c815c6f68858af69d0ccbe54"
}
     */
    /*
    {
    "access_token": "Atza|IwEBIBNp7J72AS5WkMjUq5LrhVMKk-dc9cbqoYnetxxTClYxYQb75c9x8R__m156V3fZj3UifdVq0VHsMINDGAnfXPKloDtx05sIl2EQ0Znetz8283UsCuObApHz-JY6wD2g_sZNcENHNZS_FLAB1nbWxI-c8teMv5W4ryBOlb4pMsQymSbqDfTtaPeq4RZ98vt3hzq6Zep6b7E2dyNXdcNLqbO8TI1q7SXy2-NPzGisq9CwxreVqeP9DZb91uSZs060Ofu5NneABx3SyUMa-yPWPJZ7Wqys-V472un5v9s_b--Bibz8mqWqm6OI4SV7aprOPaI_vtM92L55bxoLnzsfCNxp",
    "refresh_token": "Atzr|IwEBIITKfZoBv6fJBrBBG1tJHhgCxqePUah2uQpaUaCRgMpzYowNrpO1fQZqJuH-YxtXue5E6-0fWlQWKr9BfxuyfnCejJtaX3jxKo0TdN6l6woAv8awQrhLJuwO-KN3kDrdiB3VKyEDy-R6V0pKz7b9w0gzTN-PEL-e7uA-5CaRLOwV_299HF4ZTsz5nrfZIQMZ1S0Rgbs3TNHhw48OAlLOIXlwt95mPL7gMLvy6CJf-u5_vwh38sCQLu472ViwGGr936lGN38QSBaXaUf6mKt9LT_12C5x68nBGNsi1ZhzdMGDLJZcVHiq5CSXzL-HSa1G2VDv-0ej1YupcjBefCFBGnIS",
    "token_type": "bearer",
    "expires_in": 3600
     */
}
